package com.simuwang.daq.components.report.parser;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ReflectUtil;
import cn.hutool.core.util.StrUtil;
import com.simuwang.base.common.conts.Constants;
import com.simuwang.base.mapper.EmailFieldMappingMapper;
import com.simuwang.base.pojo.dos.EmailFieldMappingDO;
import com.simuwang.base.pojo.dto.report.ReportData;
import com.smppw.common.pojo.ValueLabelVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

public abstract class AbstractReportParser<T extends ReportData> implements ReportParser<T> {
    protected final Logger logger = LoggerFactory.getLogger(this.getClass());

    private final EmailFieldMappingMapper fieldMappingMapper;
    /**
     * 字段匹配规则
     */
    protected List<ValueLabelVO> fieldMapper;

    public AbstractReportParser(EmailFieldMappingMapper fieldMappingMapper) {
        this.fieldMappingMapper = fieldMappingMapper;
    }

    @Override
    public void init() {
        List<EmailFieldMappingDO> emailFieldMapping = this.fieldMappingMapper.getEmailFieldMapping();
        if (CollUtil.isEmpty(emailFieldMapping)) {
            this.logger.error("未设置报告解析规则！");
            return;
        }
        this.fieldMapper = emailFieldMapping.stream().map(e -> new ValueLabelVO(e.getCode(), e.getName())).collect(Collectors.toList());
    }

    /**
     * 对象字段设置
     *
     * @param extInfoMap 名称与值的对应关系
     * @param info       待设置的对象
     */
    protected void buildInfo(Map<String, Object> extInfoMap, Object info) {
        for (Map.Entry<String, Object> entry : extInfoMap.entrySet()) {
            String k = entry.getKey();
            Object v = entry.getValue();
            String fieldValue = StrUtil.toStringOrNull(v);
            if (fieldValue.startsWith("-") || fieldValue.endsWith("-")) {
                fieldValue = null;
            }
            if (fieldValue != null) {
                fieldValue = fieldValue.replace("\r", Constants.EMPTY);
            }
            for (ValueLabelVO vo : this.fieldMapper) {
                String fieldName = vo.getValue();
                List<String> labels = StrUtil.split(vo.getLabel(), ",");
                if (labels.contains(k)) {
                    try {
                        ReflectUtil.setFieldValue(info, fieldName, fieldValue);
                    } catch (Exception e) {
                        this.logger.warn("{} 字段值设置错误：{}", fieldName, e.getMessage());
                    }
                    break;
                }
                for (String label : labels) {
                    if (k.contains(label)) {
                        try {
                            ReflectUtil.setFieldValue(info, fieldName, fieldValue);
                        } catch (Exception e) {
                            this.logger.warn("{} 字段值设置错误：{}", fieldName, e.getMessage());
                        }
                        break;
                    }
                }
            }
        }
    }
}
